package main

import (
	"compress/gzip"
	"crypto/tls"
	"fmt"
	"io"
	"log"
	"math/rand"
	"net/http"
	"strings"

	"github.com/andybalholm/brotli"
)

var host map[string]([]string)

func main() {
	initHost()
	http.HandleFunc("/", nyaa)
	http.ListenAndServe("127.0.0.1:22222", nil)
}

func nyaa(w http.ResponseWriter, r *http.Request) {
	trueHost := "nyaa.si"
	tr := &http.Transport{
		TLSClientConfig: &tls.Config{InsecureSkipVerify: true},
	}
	client := &http.Client{
		CheckRedirect: func(req *http.Request, via []*http.Request) error {
			return http.ErrUseLastResponse
		},
		Transport: tr,
	}

	newUrl := r.URL
	newUrl.Host = getIPbyHost(trueHost) // IP
	newUrl.Scheme = "https"

	req, err := http.NewRequest("GET", newUrl.String(), nil)
	if err != nil {
		fmt.Println(`Error On NewRequest`)
		return
	}
	req.Header = r.Header
	req.Method = r.Method
	req.Body = r.Body   //
	req.Host = trueHost //Host

	resp, err := client.Do(req)
	if err != nil {
		fmt.Println(`Error On Do Request`)
		return
	}
	defer resp.Body.Close()

	var reader io.ReadCloser
	switch resp.Header.Get("Content-Encoding") {
	case "gzip":
		reader, err = gzip.NewReader(resp.Body)
		if err != nil {
			// fmt.Println("gzip!", "\n")
			log.Fatal("gzip")
		}
		defer reader.Close()
	case "br":
		r := brotli.NewReader(resp.Body)
		if r == nil {
			log.Fatal("error decoding br response", r)
		}
		contentType := resp.Header.Get("Content-Type")
		w.Header().Set(`Transfer-Encoding`, `br`)
		w.Header().Set("Content-Type", contentType)

		for k, v := range resp.Header {
			if k == `Content-Encoding` || k == `Transfer-Encoding` {
				continue
			}
			w.Header().Set(k, v[0])
		}

		io.Copy(w, r)
		return

	default:
		reader = resp.Body
	}
	buf := new(strings.Builder)
	io.Copy(buf, reader)
	s := buf.String()

	statusCode := resp.StatusCode
	contentType := resp.Header.Get("Content-Type")
	w.Header().Set("Content-Type", contentType)
	// w.Header().Set(`Transfer-Encoding`, `plain`)

	for k, v := range resp.Header {
		// fmt.Println(k, v)
		if k == `Content-Encoding` || k == `Transfer-Encoding` {
			// fmt.Println(k)
			continue
		}
		w.Header().Set(k, v[0])
		// for _, vv := range v {
		// w.Header().Add(k, vv)
		// }
	}
	// w.Header().Del(`Transfer-Encoding`)
	// w.Header().Set(`Transfer-Encoding`, `plain`)
	if resp.Header.Get("Set-Cookie") != "" {
		setCookie := resp.Header.Get("Set-Cookie")
		setCookie = strings.Replace(setCookie, `; secure`, "", -1)
		setCookie = strings.Replace(setCookie, `; Secure`, "", -1)
		setCookie = strings.Replace(setCookie, `Domain=.nyaa.si;`, "", -1)
		w.Header().Set("Set-Cookie", setCookie)
	}
	w.WriteHeader(statusCode)
	// fmt.Println(s)
	// fmt.Println(r.URL.Path)
	if r.URL.Path == `/` {
		s = strings.Replace(s, `</nav>`, `</nav><iframe width="100%" height="48" src="https://moonchan.xyz/nyaa.html">admin@moonchan.xyz</iframe>`, 1)
	}
	s = strings.Replace(s, `href="/static`, `href="https://sukebei.moonchan.xyz/static`, -1)

	w.Write([]byte(s))

	// w.WriteHeader(statusCode)
}

func getIPbyHost(s string) string {
	return host[s][rand.Intn(len(host[s]))]
}

func initHost() {
	host = make(map[string]([]string))
	host["nyaa.si"] = []string{
		"185.178.208.182",
	}
	host["sukebei.nyaa.si"] = []string{
		"198.251.89.38",
	}
}
